Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func Float64ToStr ¶
Float64ToStr Convert type float64 to string
func MemHash ¶
MemHash is the hash function used by go map, it utilizes available hardware instructions(behaves as aeshash if aes instruction is available). NOTE: The hash seed changes for every process. So, this cannot be used as a persistent hash.
func StrToFloat64 ¶
StrToFloat64 convert type string to float64
Types ¶
type Murmur128 ¶
type Murmur128 struct {
// contains filtered or unexported fields
}
func NewMurmur128 ¶
func NewMurmur128() *Murmur128
func (*Murmur128) EncodeSum128 ¶
Click to show internal directories.
Click to hide internal directories.